package net.doubledoordev.pay2spawn.types.guis.shapes;

import net.doubledoordev.pay2spawn.types.guis.StructureTypeGui;
import net.doubledoordev.pay2spawn.util.shapes.PointI;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;

import static net.doubledoordev.pay2spawn.types.StructureType.BLOCKDATA_KEY;
import static net.doubledoordev.pay2spawn.util.Constants.GSON;
import static net.doubledoordev.pay2spawn.util.Constants.JSON_PARSER;
import static net.doubledoordev.pay2spawn.util.shapes.AbstractShape.CENTER_KEY;
import static net.doubledoordev.pay2spawn.util.shapes.AbstractShape.REPLACEABLEONLY_KEY;
import static net.doubledoordev.pay2spawn.util.shapes.Box.*;

/**
 * @author Dries007
 */
public class PointIGui extends ShapeGuiBase
{
    public JTextField    sizeXTextField;
    public JTextField    sizeYTextField;
    public JTextField    sizeZTextField;
    public JRadioButton  noReplaceableRadioButton;
    public JRadioButton  replaceableRadioButton;
    public JRadioButton  randomReplaceableRadioButton;
    public JScrollPane   scrollPane;
    public JTextPane     jsonPane;
    public JList<String> blockList;
    public JButton       addBlockTypeButton;
    public JButton       removeBlockTypebtn;
    public JButton       parseFromJsonButton;
    public JButton       updateJsonButton;
    public JButton       saveButton;
    public JPanel        panel1;

    public PointIGui(int index, JsonObject jsonObject, StructureTypeGui callback, HashMap<String, String> typeMap)
    {
        super(index, jsonObject, PointI.class, callback, typeMap);
        init();
    }

    @Override
    public void readJson()
    {
        if (!data.has(CENTER_KEY)) data.add(CENTER_KEY, new JsonObject());

        sizeXTextField.setText(readValue(X_KEY, data));
        sizeYTextField.setText(readValue(Y_KEY, data));
        sizeZTextField.setText(readValue(Z_KEY, data));

        String replaceable = readValue(REPLACEABLEONLY_KEY, data);
        noReplaceableRadioButton.setSelected(replaceable.equals(FALSE_BYTE) || replaceable.equals(""));
        replaceableRadioButton.setSelected(replaceable.equals(TRUE_BYTE));
        randomReplaceableRadioButton.setSelected(replaceable.startsWith(RANDOM_BOOLEAN));

        blockData = data.getAsJsonArray(BLOCKDATA_KEY);

        blockList.updateUI();
        jsonPane.setText(GSON.toJson(data));
    }

    @Override
    public void updateJson()
    {
        if (!data.has(CENTER_KEY)) data.add(CENTER_KEY, new JsonObject());

        storeValue(X_KEY, data, sizeXTextField.getText());
        storeValue(Y_KEY, data, sizeYTextField.getText());
        storeValue(Z_KEY, data, sizeZTextField.getText());

        if (replaceableRadioButton.isSelected()) storeValue(REPLACEABLEONLY_KEY, data, TRUE_BYTE);
        if (noReplaceableRadioButton.isSelected()) storeValue(REPLACEABLEONLY_KEY, data, FALSE_BYTE);
        if (randomReplaceableRadioButton.isSelected()) storeValue(REPLACEABLEONLY_KEY, data, RANDOM_BOOLEAN);

        data.add(BLOCKDATA_KEY, blockData);

        blockList.updateUI();
        jsonPane.setText(GSON.toJson(data));
    }

    @Override
    public void setupListeners()
    {
        saveButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                updateJson();
                callback.callback(rewardID, data);
                dialog.dispose();
            }
        });
        parseFromJsonButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                try
                {
                    data = JSON_PARSER.parse(jsonPane.getText()).getAsJsonObject();
                    readJson();
                    jsonPane.setForeground(Color.black);
                }
                catch (Exception e1)
                {
                    jsonPane.setForeground(Color.red);
                    e1.printStackTrace();
                }
            }
        });
        updateJsonButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                updateJson();
            }
        });
        addBlockTypeButton.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                new BlockDataGui(-1, new JsonObject(), instance);
            }
        });
        blockList.addMouseListener(new MouseAdapter()
        {
            @Override
            public void mouseClicked(MouseEvent e)
            {
                if (e.getClickCount() == 2)
                {
                    new BlockDataGui(blockList.getSelectedIndex(), blockData.get(blockList.getSelectedIndex()).getAsJsonObject(), instance);
                }

                removeBlockTypebtn.setEnabled(!blockList.isSelectionEmpty());
            }
        });
        removeBlockTypebtn.addActionListener(new ActionListener()
        {
            @Override
            public void actionPerformed(ActionEvent e)
            {
                int selectedIndex = blockList.getSelectedIndex();
                if (selectedIndex != -1)
                {
                    JsonArray newBlockData = new JsonArray();
                    for (int i = 0; i < blockData.size(); i++) if (i != selectedIndex) newBlockData.add(blockData.get(i));
                    blockData = newBlockData;
                }
                updateJson();
                removeBlockTypebtn.setEnabled(!blockList.isSelectionEmpty());
            }
        });
        blockList.setModel(getBlockDataModel());
    }

    @Override
    public JPanel getPanel()
    {
        return panel1;
    }

    {
        // GUI initializer generated by IntelliJ IDEA GUI Designer
        // >>> IMPORTANT!! <<<
        // DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$()
    {
        panel1 = new JPanel();
        panel1.setLayout(new GridBagLayout());
        final JPanel panel2 = new JPanel();
        panel2.setLayout(new GridBagLayout());
        GridBagConstraints gbc;
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        panel1.add(panel2, gbc);
        final JLabel label1 = new JLabel();
        label1.setText("X: ");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(label1, gbc);
        sizeXTextField = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel2.add(sizeXTextField, gbc);
        final JLabel label2 = new JLabel();
        label2.setText("INT");
        gbc = new GridBagConstraints();
        gbc.gridx = 4;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(label2, gbc);
        final JLabel label3 = new JLabel();
        label3.setText("Y: ");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(label3, gbc);
        sizeYTextField = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel2.add(sizeYTextField, gbc);
        final JLabel label4 = new JLabel();
        label4.setText("INT");
        gbc = new GridBagConstraints();
        gbc.gridx = 4;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(label4, gbc);
        final JLabel label5 = new JLabel();
        label5.setText("Z: ");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(label5, gbc);
        sizeZTextField = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 2;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel2.add(sizeZTextField, gbc);
        final JLabel label6 = new JLabel();
        label6.setText("INT");
        gbc = new GridBagConstraints();
        gbc.gridx = 4;
        gbc.gridy = 2;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(label6, gbc);
        noReplaceableRadioButton = new JRadioButton();
        noReplaceableRadioButton.setText("All blocks");
        gbc = new GridBagConstraints();
        gbc.gridx = 2;
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(noReplaceableRadioButton, gbc);
        replaceableRadioButton = new JRadioButton();
        replaceableRadioButton.setText("Replaceable only");
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(replaceableRadioButton, gbc);
        randomReplaceableRadioButton = new JRadioButton();
        randomReplaceableRadioButton.setText("Random");
        gbc = new GridBagConstraints();
        gbc.gridx = 3;
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.WEST;
        panel2.add(randomReplaceableRadioButton, gbc);
        final JPanel panel3 = new JPanel();
        panel3.setLayout(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 6;
        gbc.gridwidth = 5;
        gbc.weightx = 1.0;
        gbc.weighty = 0.5;
        gbc.fill = GridBagConstraints.BOTH;
        panel2.add(panel3, gbc);
        final JLabel label7 = new JLabel();
        label7.setText("Json:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        panel3.add(label7, gbc);
        scrollPane = new JScrollPane();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        panel3.add(scrollPane, gbc);
        jsonPane = new JTextPane();
        jsonPane.setEnabled(true);
        jsonPane.setText("");
        jsonPane.setToolTipText("Make sure you hit \"Parse from JSON\" after editing this!");
        scrollPane.setViewportView(jsonPane);
        final JScrollPane scrollPane1 = new JScrollPane();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.gridwidth = 5;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        panel2.add(scrollPane1, gbc);
        blockList = new JList();
        scrollPane1.setViewportView(blockList);
        final JPanel panel4 = new JPanel();
        panel4.setLayout(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 5;
        gbc.gridwidth = 5;
        gbc.fill = GridBagConstraints.BOTH;
        panel2.add(panel4, gbc);
        addBlockTypeButton = new JButton();
        addBlockTypeButton.setText("Add blockType");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.weightx = 0.5;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel4.add(addBlockTypeButton, gbc);
        removeBlockTypebtn = new JButton();
        removeBlockTypebtn.setText("Remove selected blockType");
        gbc = new GridBagConstraints();
        gbc.gridx = 2;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.weightx = 0.5;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel4.add(removeBlockTypebtn, gbc);
        final JPanel panel5 = new JPanel();
        panel5.setLayout(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.BOTH;
        panel1.add(panel5, gbc);
        parseFromJsonButton = new JButton();
        parseFromJsonButton.setText("Parse from Json");
        parseFromJsonButton.setToolTipText("Push the button!");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel5.add(parseFromJsonButton, gbc);
        updateJsonButton = new JButton();
        updateJsonButton.setText("Update Json");
        updateJsonButton.setToolTipText("Push the button!");
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel5.add(updateJsonButton, gbc);
        saveButton = new JButton();
        saveButton.setText("Save");
        gbc = new GridBagConstraints();
        gbc.gridx = 2;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        panel5.add(saveButton, gbc);
        ButtonGroup buttonGroup;
        buttonGroup = new ButtonGroup();
        buttonGroup.add(replaceableRadioButton);
        buttonGroup.add(noReplaceableRadioButton);
        buttonGroup.add(randomReplaceableRadioButton);
    }

    /** @noinspection ALL */
    public JComponent $$$getRootComponent$$$()
    { return panel1; }
}
